<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Mail
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.5 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Mail.php,v 1.5 2005/04/07 10:14:55 mike Exp $
 */

/**
 * Requires PEAR
 */
class_exists('PEAR') or require_once 'PEAR.php';

/**
 * Gadget_Mail Instances
 * @ignore
 */
$GLOBALS['_Gadget']['Mail']['Instances'] = array();

/**
 * Gadget_Mail
 * 
 * Gadget::Mail is an yet ordinary though sophisticated interface to IMAP
 * Mailboxes, which supports all common tasks such as folder listing and
 * traversal as well as message rewtreival and storage.  The companion
 * classes Gadget_Mail_Message and Gadget_Mail_Attechment operate as
 * simple interfaces to messages and mail attachments.
 *
 * @package     Gadget
 * @subpackage  Gadget_Mail
 * @since       1.0
 */
class Gadget_Mail
{
    /**
     * Account (someone@example.com)
     * 
     * @var     string
     * @access  public
     */
    var $account;
    
    /**
     * Password
     * 
     * @var     string
     * @access  public
     */
    var $password;
    
    /**
     * Connected
     * 
     * @var     bool
     * @access  public
     */
    var $connected;
    
    /**
     * Options
     * 
     * @var     array
     * @access  protected
     */
    var $options = array(
        'host'  => 'localhost',
        'port'  => 143,
        'proto' => 'imap',
        'ssl'   => '',
        'folder'=> 'INBOX',
        'list'  => 0,
    );
    
    /**
     * Folders
     * 
     * @var     array
     * @access  protected
     */
    var $folders = array();
    
    /**
     * Headers
     * 
     * @var     array
     * @access  protected
     */
    var $headers = array(
        'INBOX' => array(),
    );
    
    /**
     * Messages
     * 
     * @var     array
     * @access  protected
     */
    var $messages = array(
        'INBOX' => array(),
    );
    
    /**
     * IMAP Resource
     * 
     * @var     resource
     * @access  protected
     */
    var $imap;
    
    /**
     * Constructor
     * 
     * @access  public
     * @param   string  $account
     * @param   string  $password
     * @param   array   $options
     */
    function Gadget_Mail($account = null, $password = null, $options = array())
    {
        $this->account = $account;
        $this->password = $password;
        $this->options['list'] = SA_MESSAGES | SA_RECENT | SA_UNSEEN;
        $this->setOptions($options);
    }
    
    /**
     * Set Options
     * 
     * @access  public
     * @return  void
     * @param   array   $options
     */
    function setOptions($options)
    {
        foreach ((array) $options as $o => $v) {
            if (is_array($v)) {
                if (isset($this->options[$o]) && is_array($this->options[$o])) {
                    $this->options[$o] = array_merge($this->options[$o], $v);
                } else {
                    $this->options[$o] = $v;
                }
            } else {
                $this->options[$o] = $v;
            }
        }
    }
    
    /**
     * Singleton
     * 
     * @static
     * @access  public
     * @return  object  Gadget_Mail
     * @param   string  $account
     * @param   string  $passwd
     * @param   array   $options
     */
    function &singleton($account, $passwd, $options = array())
    {
        $id = md5(serialize(array($account, $password, $options)));
        if (!isset($GLOBALS['_Gadget']['Mail']['Instances'][$id])) {
            $GLOBALS['_Gadget']['Mail']['Instances'][$id] =
                &new Gadget_Mail($account, $password, $options);
        }
        return $GLOBALS['_Gadget']['Mail']['Instances'][$id];
    }
    
    /**
     * Get Connection String
     * 
     * @access  public
     * @return  string
     * @param   string  $folder
     * @param   string  $append
     */
    function connectionString($folder = '', $append = '')
    {
        return sprintf('{%s:%d/%s%s%s}',
            $this->options['host'],
            $this->options['port'],
            $this->options['proto'],
            $this->options['ssl'] ? '/'. $this->options['ssl'] :'',
            $append
        ) . $folder;
    }
    
    /**
     * Raise Error
     * 
     * @access  protected
     * @return  object  PEAR_Error
     * @param   string  $error
     * @param   int     $code
     * @param   bool    $le
     */
    function raiseError($error, $code = 0, $le = false)
    {
        return PEAR::raiseError($error, $code, null, null, $le ? imap_last_error() : null);
    }
    
    /**
     * Encode UTF-7
     * 
     * @access  public
     * @return  string
     * @param   string  $string
     * @param   string  $from
     */
    function encodeUTF7($string, $from = GADGET_CHARSET)
    {
        $utf_7 = null;
        $shift = null;
        
        if (strcasecmp($from, 'UTF-8')) {
            $string = iconv($from, 'UTF-8', $string);
        }
        
        for ($i = 0, $l = strlen($string); $i < $l; $i++) {
            $char = $string{$i};
            
            // special case for "&" char
            if ($char === '&') {
                $utf_7 .= '&-';
            }
            
            // non ASCII-printable chars
            elseif ($char <= "\x1f" || $char >= "\x7f") {
                if (!isset($shift)) {
                    $utf_7 .= '&';
                }
                
                // handle 2-byte UTF-8
                if ($char >= "\xc2") {
                
                    if (($dec = ord($string{++$i})) % 64) {
                        $shift .= "\0". chr($dec + (ord($char) % 0xc2) * 64);
                    } else {
                        $shift .= "\0". $string{$i};
                    }
                   
                } else {
                    $shift .= $char;
                }
                
            }
            
            // encode shifted chars
            else {
                if (isset($shift)) {
                    $utf_7 .= str_replace('/', ',', rtrim(base64_encode($shift), '=')) . '-';
                    $shift = null;
                }
                $utf_7 .= $char;
            }
        }
        
        // still in shift mode?
        if (isset($shift)) {
            $utf_7 .= str_replace('/', ',', rtrim(base64_encode($shift), '=')) . '-';
        }
        
        return $utf_7;
    }
    
    /**
     * Decode UTF-7
     * 
     * @access  public
     * @return  string
     * @param   string  $utf_7
     * @param   string  $to
     */
    function decodeUTF7($utf_7, $to = GADGET_CHARSET)
    {
        $strng = null;
        $shift = null;
        
        for ($i = 0, $l = strlen($utf_7); $i < $l; $i++) {
            $char = $utf_7{$i};
            
            // in shift mode?
            if (isset($shift)) {
                
                // stepping out of shift mode?
                if ($char === '-') {
                    
                    // just the "&" char?
                    if ($shift === '') {
                        $strng .= '&';
                    } else {
                        $data = base64_decode(str_replace(',', '/', $shift));
                        for ($i2 = 0, $l2 = strlen($data); $i2 < $l2; $i2++) {
                            if ($data{$i2} == "\0") {
                                $hex = ord($data{$i2+1});
                                $data{$i2}   = chr(192 + (($hex - ($hex % 64)) / 64));
                                $data{++$i2} = chr(128 + ($hex % 64));
                            }
                        }
                        $strng .= $data;
                    }
                    $shift = null;
                }
                
                // append shifted char
                else {
                    $shift .= $char;
                }
            }
            
            // the "&" char - step into shift mode
            elseif ($char === '&') {
                $shift = '';
            }
            
            // normal char
            else {
                $strng .= $char;
            }
        }
        return iconv('UTF-8', $to, $strng);
    }
    
    /**
     * Mime Helper
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string
     */
    function decodeMIME($mime)
    {
        return preg_replace(
            '/=\?x-unknown\?B\?(.*?)\?=/e', 
            'base64_decode(\'\1\')', 
            iconv('utf-8', GADGET_CHARSET, imap_utf8($mime))
        );
    }
    
    /**
     * Connect
     * 
     * @access  protected
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $folder
     */
    function connect($folder = null)
    {
        if ($this->connected && is_resource($this->imap)) {
            return true;
        }
        
        $this->imap = @imap_open(
            $this->connectionString(isset($folder) ? $folder : $this->options['folder']), 
            $this->account, 
            $this->password
        );
        
        if (!is_resource($this->imap)) {
            return $this->raiseError(
                'Couldn\'t connect to '. $this->connectionString(), 0, true
            );
        }
        
        return $this->connected = true;
    }
    
    /**
     * Disconnect
     * 
     * @access  public
     * @return  void
     */
    function disconnect()
    {
        if ($this->connected && is_resource($this->imap)) {
            @imap_close($this->imap);
            $this->imap = null;
        }
        $this->connected = false;
    }
    
    /**
     * List Folders
     * 
     * @access  public
     * @return  mixed   Returns array of folders on success or PEAR_Error on failure.
     * @param   bool    $refresh
     */
    function listFolders($refresh = false)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        
        $connStr = $this->connectionString();
        
        if ($refresh || !count($this->folders)) {
            
            // call imap_status on current folder
            $this->folders[$this->options['folder']] = 
                imap_status($this->imap, $connStr, $this->options['list']);
            
            // list folders
            $folders = @imap_list($this->imap, $connStr, '*');
            
            foreach ((array) $folders as $f) {
                // strip hostinfo
                $folder = $this->decodeUtf7(str_replace($this->connectionString(), '', $f));
                
                // save imap_status of folders in this folder
                $this->folders[$folder] = 
                    imap_status($this->imap, $f, $this->options['list']);
            }
        }
        
        return $this->folders;
    }
    
    /**
     * Count All Messages
     * 
     * @access  public
     * @return  mixed   Returns count of messages in the specified or current
     *                  folder on success or PEAR_Error on failure.
     * @param   string  $folder
     * @param   bool    $refresh
     */
    function countAll($folder = null, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if (!isset($folder)) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        return $this->folders[$folder]->messages;
    }
    
    /**
     * Count New Messages
     * 
     * @access  public
     * @return  mixed   Returns count of new messages in the specified or 
     *                  current folder on success or PEAR_Error on failure.
     * @param   string  $fodler
     * @param   bool    $refresh
     */
    function countNew($folder = null, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if (!isset($folder)) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        return $this->folders[$folder]->unseen;
    }
    
    /**
     * Count Recent Messages
     * 
     * @access  public
     * @return  mixed   Returns count of recent messages in the specified or
     *                  current folder on success or PEAR_Error on failure.
     * @param   string  $folder
     * @param   bool    $refresh
     */
    function countRecent($folder = null, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if (!isset($folder)) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        return $this->folders[$folder]->recent;
    }
    
    /**
     * Get Quota
     * 
     * @access  public
     * @return  mixed   Returns array of quotas (size/msgs) of the specified or
     *                  current folder on success or PEAR_Error on failure.
     * @param   string  $folder
     * @param   bool    $refresh
     */
    function getQuota($folder = null, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if (!isset($folder)) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        return @imap_get_quotaroot($this->imap, $folder);
    }
    
    /**
     * Set Folder
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $folder
     * @param   bool    $refresh
     */
    function setFolder($folder, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if ($this->options['folder'] == $folder) {
            return true;
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        $mailbox = $this->connectionString($this->encodeUtf7($folder));
        if (!@imap_reopen($this->imap, $mailbox)) {
            return $this->raiseError(
                "Couldn't set current Folder to '$folder'", 0, true
            );
        }
        $this->options['folder'] = $folder;
        return true;
    }
    
    
    /**
     * Get Folder
     * 
     * @access  public
     * @return  mixed   Returns the object of the specified or current folder
     *                  on success or PEAR_Error on failure.
     * @param   string  $folder
     * @param   bool    $refresh
     */
    function getFolder($folder = null, $refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        if (!isset($folder)) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        return $this->folders[$folder];
    }
    
    /**
     * Current Folder
     * 
     * @access  public
     * @return  mixed   Returns name of current folder on success or PEAR_Error
     *                  on failure.
     * @param   bool    $refresh
     */
    function currentFolder($refresh = false)
    {
        if (PEAR::isError($e = $this->listFolders($refresh))) {
            return $e;
        }
        return $this->options['folder'];
    }
    
    /**
     * Get Sorted Ids
     * 
     * @access  public
     * @return  mixed   Returns array of sorted message ids on success or
     *                  PEAR_Error on failure.
     * @param   int     $sort
     * @param   int     $reverse
     */
    function getSortedIds($sort = SORTDATE, $reverse = 1)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        if (!is_array($sorted = @imap_sort($this->imap, $sort, $reverse))) {
            return PEAR::raiseError('Could not sort messages', 0, true);
        }
        return $sorted;
    }
    
    /**
     * Get Threaded Ids
     * 
     * @access  public
     * @return  mixed   Returns array of threaded (by References header) message
     *                  ids on success or PEAR_Error on failure.
     * @param   int     $flags
     */
    function getThreadedIds($flags = null)
    {
        if (PEAR::isError($e = $this->_connect())) {
            return $e;
        }
        
        isset($flags) or $flags = SE_FREE | SE_NOPREFETCH;
        
        if (!is_array($thread = @imap_thread($this->imap, $flags))) {
            return $this->raiseError('Could not generate thread', 0, true);
        }
        
        return $thread;
    }
    
    /**
     * List Headers
     * 
     * @access  public
     * @return  mixed   Returns list of headers of the current folder on success
     *                  or PEAR_Error on failure.
     * @param   int     $sort
     * @param   int     $reverse
     * @param   bool    $refresh
     */
    function listHeaders($sort = SORTDATE, $reverse = 1, $offset = 0, $limit = -1, $refresh = false)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        
        if ($refresh || empty($this->headers[$this->options['folder']])) {
            $this->headers[$this->options['folder']] = array();
            
            // fetch headers for all messages in this folder
            if (PEAR::isError($sorted = $this->getSortedIds($sort, $reverse))) {
                return $sorted;
            }
            foreach ($sorted as $i) {
                $this->headers[$this->options['folder']][$i] = 
                    imap_header($this->imap, $i);
            }
        }
        
        return $this->headers[$this->options['folder']];
    }
    
    /**
     * Get Message
     * 
     * @access  public
     * @return  mixed   Returns requested Gadget_Mail_Message on success or 
     *                  PEAR_Error on failure.
     * @param   int     $id
     * @param   bool    $refresh
     */
    function &getMessage($id, $refresh = false)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        
        if ($refresh || !isset($this->messages[$this->options['folder']][$id])) {
            class_exists('Gadget_Mail_Message') or 
                require_once 'Gadget/Mail/Message.php';
            $this->messages[$this->options['folder']][$id] = 
                &Gadget_Mail_Message::fetch($this->imap, $id);
        }
        
        return $this->messages[$this->options['folder']][$id];
    }
    
    /**
     * Get Headers
     * 
     * @access  public
     * @return  mixed   Returns requested message headers on success or
     *                  PEAR_Error on failure.
     * @param   int     $id
     * @param   bool    $refresh
     */
    function getHeaders($id, $refresh = false)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        
        if ($refresh || !isset($this->headers[$this->options['folder']][$id])) {
            if ($headers = @imap_header($this->imap, $id)) {
                $this->headers[$this->options['folder']][$id] = $headers;
            } else {
                return $this->raiseError('Could not fetch message headers', 0, true);
            }
        }
        
        return $this->headers[$this->options['folder']][$id];
    }
    
    /**
     * Create Folder
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $name
     * @param   string  $in
     */
    function createFolder($name, $in = 'INBOX')
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        $folder = $in ? $in .'.'. $name : $name;
        $folder = $this->connectionString($this->encodeUtf7($folder));
        if (!@imap_createmailbox($this->imap, $folder)) {
            return $this->raiseError("Couldn't create folder '$name' in '$in'", 0, true);
        }
        return true;
    }
    
    /**
     * Delete Folder
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $name
     * @param   string  $in
     */
    function deleteFolder($name, $in = 'INBOX')
    {
        if (PEAR::isError($e = $this->listFolders())) {
            return $e;
        }
        $folder = $in ? $in .'.'. $name : $name;
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$name' doesn't exist in '$in'");
        }
        $folder = $this->connectionString($this->encodeUtf7($folder));
        if (!@imap_deletemailbox($this->imap, $folder)) {
            return $this->raiseError("Couldn't delete folder '$folder'", 0, true);
        }
        return true;
    }
    
    /**
     * Subscribe to Folder
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $name
     */
    function subscribeFolder($name, $in = 'INBOX')
    {
        if (PEAR::isError($e = $this->listFolders())) {
            return $e;
        }
        $folder = $in ? $in .'.'. $name : $name;
        $folder = $this->connectionString($this->encodeUtf7($folder));
        if (!@imap_subscribe($this->imap, $folder)) {
            return $this->raiseError("Couldn't subscribe to folder '$folder'", 0, true);
        }
        return true;
    }
    
    /**
     * Delete Message
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   int     $id
     * @param   bool    $trash
     */
    function deleteMessage($id, $trash = true)
    {
        if (PEAR::isError($e = $this->connect())) {
            return $e;
        }
        
        if ($trash) {
            if (PEAR::isError($e = $this->listFolders())) {
                return $e;
            }
            if (!isset($this->folders['INBOX.Trash'])) {
                if (PEAR::isError($e = $this->createFolder('Trash'))) {
                    return $e;
                }
            }
            if (PEAR::isError($e = $this->moveMessage($id, 'INBOX.Trash'))) {
                return $e;
            }
        }
        
        if (!@imap_delete($this->imap, $id)) {
            return $this->raiseError("Couldn't delete message '$id'", 0, true);
        }
        return true;
    }
    
    /**
     * Move Message
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   int     $id
     * @param   string  $folder
     */
    function moveMessage($id, $folder)
    {
        if (PEAR::isError($e = $this->listFolders())) {
            return $e;
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        if ($folder == $this->currentFolder()) {
            return $this->raiseError('Cannot move messages within one folder');
        }
        if (!@imap_mail_move($this->imap, $id, $this->encodeUtf7($folder))) {
            return $this->raiseError("Couldn't move message(s) '$id'", 0, true);
        }
        @imap_expunge($this->imap);
        return true;
    }
    
    /**
     * Expunge
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $folder
     */
    function expungeFolder($folder = null)
    {
        if (PEAR::isError($e = $this->setFolder($folder))) {
            return $e;
        }
        return imap_expunge($this->imap);
    }
    
    /**
     * Empty
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $folder
     */
    function emptyFolder($folder = null)
    {
        if (PEAR::isError($e = $this->listFolders())) {
            return $e;
        }
        
        if (isset($folder)) {
            if ($folder != $this->currentFolder()) {
                if (PEAR::isError($e = $this->setFolder($folder))) {
                    return $e;
                }
            }
        }
        
        foreach (range(1, $this->countAll()) as $id) {
            if (!@imap_delete($this->imap, $id)) {
                return $this->raiseError("Couldn't delete message '$id'", 0, true);
            }
        }
        
        return @imap_expunge($this->imap);
    }
    
    /**
     * Send Message
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   object  Gadget_Mail_Message $message
     * @param   string  $mode
     */
    function sendMessage(&$message)
    {
        $params = Gadget::getConf('Mail');
        if (PEAR::isError($e = $message->send($params['driver'], $params))) {
            return $e;
        }
        return $this->storeMessage($message->toString(), 'INBOX.Sent');
    }
    
    /**
     * Store Message
     * 
     * @access  public
     * @return  mixed   Returns true on success or PEAR_Error on failure.
     * @param   string  $rawmessage
     * @param   string  $folder
     * @param   string  $flags
     */
    function storeMessage($rawmessage, $folder = 'INBOX.Drafts', $flags = '\\Seen')
    {
        if (PEAR::isError($e = $this->listFolders())) {
            return $e;
        }
        if (!$folder) {
            $folder = $this->options['folder'];
        }
        if (!isset($this->folders[$folder])) {
            return $this->raiseError("Folder '$folder' doesn't exist");
        }
        $folder = $this->connectionString($this->encodeUtf7($folder));
        if (!@imap_append($this->imap, $folder, $rawmessage, $flags)) {
            return $this->raiseError('Couldn\'t store message', 0, true);
        }
        return true;
    }
    
}

?>
