<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Preferences.php,v 1.1 2005/04/04 11:50:57 mike Exp $
 */

/**
 * Requires Gadget_Provider
 */
class_exists('Gadget_Provider') or require_once 'Gadget/Provider.php';

/**
 * Gadget_Provider_Preferences
 *
 * Gadget's default preferences provider
 * 
 * @package     Gadget
 * @subpackage  Gadget_Provider
 * @since       1.0
 */
class Gadget_Provider_Preferences extends Gadget_Provider
{
    var $provider = PROVIDER_PREFERENCES;
    var $preferences;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        if ($prefdata = Gadget::currentUser('prefdata')) {
            $this->preferences = array_merge(
                Gadget::getConf('Preferences'), 
                unserialize($prefdata)
            );
        } else {
            if (isset($_SESSION['anonprefs'])) {
                $this->preferences = $_SESSION['anonprefs'];
            } else {
                $this->preferences = Gadget::getConf('Preferences');
            }
        }
    }
    
    /**
     * Get
     * 
     * @access  public
     * @return  mixed
     * @param   string
     */
    function get($pref = null)
    {
        if (!isset($pref)) {
            return $this->preferences;
        }
        if (!isset($this->preferences[$pref])) {
            return false;
        }
        return $this->preferences[$pref];
    }
    
    /**
     * Set
     * 
     * @access  public
     * @return  mixed
     * @param   array
     */
    function set($pref)
    {
        foreach ((array) $pref as $p => $v) {
            $this->preferences[$p] = $v;
        }
        
        if ($user = Gadget::currentUser('id')) {
            $data = Gadget::DAO('preferences');
            $data->user = $user;
            $data->data = serialize($this->preferences);
            return $data->update();
        } else {
            $_SESSION['anonprefs'] = $this->preferences;
            return true;
        }
    }
    
}

?>
