<?php

class_exists('Gadget_Service') or require_once 'Gadget/Service.php';

/**
 * Gadget Webdevelopment Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.4 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Mailaccount.php,v 1.4 2005/04/08 16:19:19 mike Exp $
 */

/** 
 * Gadget_Service_Mailaccount
 *
 * Gadget Service for Mail Account Operations.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Mailaccount extends Gadget_Service
{
    var $service = SERVICE_MAILACCOUNT;
    
    var $fault;
    var $client;
    var $method;
    var $account;
    var $domain;
    
    /**
     * Init
     * 
     * @access  protected
     * @return  void
     */
    function init()
    {
        class_exists('XML_RPC_Client') or require_once 'XML/RPC.php';
        $this->domain = Gadget::getConf('Gadget', 'maildomain');
        $this->client = &new XML_RPC_Client(
            Gadget::getConf('vpop-xmlrpc', 'path'),
            Gadget::getConf('vpop-xmlrpc', 'host')
        );
        $this->client->setCredentials(
            Gadget::getConf('vpop-xmlrpc', 'user'),
            Gadget::getConf('vpop-xmlrpc', 'pass')
        );
    }
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     */
    function execute($params)
    {
        list($method, $params) = $params;
        
        if (!count($params)) {
            $this->fault = 'Empty params';
            return false;
        }

        if (empty($params['domain'])) {
            $params['domain'] = $this->domain;
        }

        $this->method  = strToLower($method);
        $this->account = $params['user'] .'@'. $params['domain'];
        
        // sane parameters?
        switch ($this->method)
        {
            case 'adduser':
                if (!isset($params['user'], $params['password'])) {
                    return false;
                }
            break;
            
            case 'deluser':
                if (!isset($params['user'])) {
                    return false;
                }
            break;
            
            case 'setquota':
                if (!isset($params['user'], $params['quota'])) {
                    return false;
                }
            break;
            
            case 'passwd':
                if (!isset($params['user'], $params['password'])) {
                    return false;
                }
            break;
            
            case 'auth':
                if (!isset($params['user'], $params['password'])) {
                    return false;
                }
            break;
            
            default:
                return false;
        }
        
        return $this->rpcScalar($this->method, $params);
    }
    
    /**
     * Change Password
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     */
    function passwd($user, $password)
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        return $this->rpcScalar('passwd', $params);
    }
    
    /**
     * Auth
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     */
    function auth($user, $password)
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        return $this->rpcScalar('auth', $params);
    }
    
    /**
     * Add User
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $password
     * @param   string  $fullname
     */
    function addUser($user, $password, $fullname = '')
    {
        $params = array('password' => $password);
        $this->userDomain($params, $user);
        
        if ($fullname) {
            $params['fullname'] = $fullname;
        } else {
            $params['fullname'] = $params['user'].'@'.$params['domain'];
        }
        
        return $this->rpcScalar('adduser', $params);
    }
    
    /**
     * Delete User
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     */
    function delUser($user)
    {
        $this->userDomain($params, $user);
        return $this->rpcScalar('deluser', $params);
    }
    
    /**
     * Set Quota
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $user
     * @param   string  $quota
     */
    function setQuota($user, $quota)
    {
        $params = array('quota' => $quota);
        $this->userDomain($params, $user);
        return $this->rpcScalar('setquota', $params);
    }
    
    /**
     * Add Domain
     * 
     * @access  public
     * @return  bool@PEAR_Error
     * @param   string  $domain
     * @param   string  $password
     */
    function addDomain($domain, $password)
    {
        return $this->rpcScalar('adddomain', 
            array('domain' => $domain, 'password' => $password));
    }
    
    /**
     * Delete Domain
     * 
     * @access  public
     * @return  bool|PEAR_Error
     * @param   string  $domain
     */
    function delDomain($domain)
    {
        return $this->rpcScalar('deldomain', array('domain' => $domain));
    }
    
    /**
     * List Users
     * 
     * @access  public
     * @return  mixed
     */
    function listUsers($domain)
    {
        return $this->rpcStruct('listusers', array('domain' => $domain));
    }
    
    /**
     * List Domains
     * 
     * @access  public
     * @return  mixed
     */
    function listDomains()
    {
        return $this->rpcStruct('listdomains');
    }
    
    /**
     * User Domain
     * 
     * @access  protected
     * @return  void
     * @param   array   &$params
     * @param   string  $user
     */
    function userDomain(&$params, $user)
    {
        if (preg_match('/(.+)@(.+)/', $user, $matches)) {
            $params['user']     = $matches[1];
            $params['domain']   = $matches[2];
        } else {
            $params['user']     = $user;
            $params['domain']   = $this->domain;
        }
    }
    
    /**
     * RPC with scalar return value
     * 
     * @access  protected
     * @return  bool|PEAR_Error
     * @param   string  $action
     * @param   array   $params
     */
    function rpcScalar($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $result->xv->scalarVal();
    }
    
    /**
     * RPC with struct return value
     * 
     * @access  protected
     * @return  array|PEAR_Error
     * @param   string  $action
     * @param   string  $params
     */
    function rpcStruct($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $this->flatten($result->xv->me['struct']);
    }
    
    /**
     * RPC with array return value
     * 
     * @access  protected
     * @return  array|PEAR_Error
     * @param   string  $action
     * @param   string  $params
     */
    function rpcArray($action, $params = array())
    {
        $result = &$this->_rpc($action, $params);
        if (PEAR::isError($result)) {
            return $result;
        }
        return $this->flatten($result->xv->me['array']);
    }
    
    /**
     * @private
     */
    function _rpc($action, $params)
    {
        $value = null;
        if (count($params)) {
            $values = array();
            foreach ((array) $params as $name => $value) {
                $values[$name] = &new XML_RPC_Value($value, 'string');
            }
            $value = array(new XML_RPC_Value($values, 'struct'));
        }
        $message = &new XML_RPC_Message("vpop.$action", $value);
        $message->setSendEncoding(GADGET_CHARSET);
        $result  = $this->client->send($message);
        if ($result->faultCode()) {
            return PEAR::raiseError($result->faultString(), $result->faultCode());
        }
        return $result;
    }
    
    /**
     * Flatten Structure
     * 
     * @access  protected
     * @return  array
     * @param   array   $struct
     */
    function flatten(&$struct)
    {
        $result = array();
        foreach ((array) $struct as $key => $val) {
            switch ($type = $val->kindOf())
            {
                case 'array';
                case 'struct':
                    $result[$key] = $this->flatten($val->me[$type]);
                break;
                
                case 'undef':
                    $result[$key] = $$val->getVal();
                break;
                
                default:
                    $result[$key] = $val->scalarVal();
            }
        }
        return $result;
    }
    
    /**
     * On Success
     * 
     * @access  protected
     * @return  bool true
     */
    function onSuccess()
    {
        return parent::onSuccess($this->method .'('. $this->account .')');
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        return parent::onFailure($this->method .'('. $this->account .') '. $this->fault);
    }
    
    /**
     * Safe Name
     * 
     * @static
     * @access  public
     * @return  string
     * @param   string  $name
     */
    function safeName($name)
    {
        return preg_replace('/[^\w._-]+/', '_', $name);
    }
}

?>
