<?php

/**
 * Gadget Webdevelopment Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Security.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/** 
 * Gadget_Service_Security
 *
 * Gadget's basic security service.  Enable with
 * <pre>
 * [Security]
 * enabled              = 1
 * session_hijacking    = 1
 * </pre>
 * in your config.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Service
 * @since       1.0
 */
class Gadget_Service_Security extends Gadget_Service
{
    var $service = SERVICE_SECURITY;
    var $condition;
    
    /**
     * Execute
     * 
     * @access  protected
     * @return  bool
     */
    function execute()
    {
        if (Gadget::getConf('Security', 'http_headers')) {
            if (!$this->_HttpHeaders()) {
                $this->condition = 'http_headers';
                return false;
            }
        }
        if ($rm = Gadget::getConf('Security', 'request_methods')) {
            if (!$this->_RequestMethods($rm)) {
                $this->condition = 'request_methods';
                return false;
            }
        }

        if (Gadget::getConf('Security', 'session_hijacking')) {
            if (!$this->_SessionHijacking()) {
                $this->condition = 'session_hijacking';
                return false;
            }
        }
        if (Gadget::getConf('Security', 'challenge_cookie')) {
            if (!$this->_ChallengeCookie()) {
                $this->condition = 'challenge_cookie';
                return false;
            }
        }
        return true;
    }
    
    /**
     * Attempt: Prevent Session Hijacking
     * 
     * @access  private
     * @return  bool
     */
    function _SessionHijacking()
    {
        if (isset($_SESSION['SECURITY']['USER_AGENT'])) {
            return $_SESSION['SECURITY']['USER_AGENT'] 
                == @$_SERVER['HTTP_USER_AGENT'];
        }
        if (isset($_SERVER['HTTP_USER_AGENT'])) {
            $_SESSION['SECURITY']['USER_AGENT'] = $_SERVER['HTTP_USER_AGENT'];
        }
        return true;
    }
    
    /**
     * Attempt: Prevent Interception through a Challenge Cookie
     * 
     * @access  private
     * @return  bool
     */
    function _ChallengeCookie()
    {
        $new = md5(uniqid('GADGET', 1));
        $usr = Gadget::getCookie('challenge', false);
        $old = &$_SESSION['SECURITY']['CHALLENGE'];
        
        // OK if first request ($old not set)
        if (!$OK = !isset($old['CURR'])) {
            // OK if challenge codes equal
            $OK = $usr === $old['CURR'];
            
            // user may have opened several pages at once
            // this tends to make the whole thingy useless
            if (!$OK && isset($old['PREV']) && TIME - $old['PREV']['TIME'] < 2) {
                $OK = $usr === $old['PREV']['CODE'];
            }
            
            $old['PREV'] = array('TIME' => TIME, 'CODE' => $old['CURR']);
        }

        Gadget::setCookie('challenge', $new, true);
        $old['CURR'] = $new;

        return $OK;
    }
    
    /**
     * Attempt: Check presence of common HTTP headers
     * 
     * @access  private
     * @return  bool
     */
    function _HttpHeaders()
    {
        if (empty($_SERVER['HTTP_HOST'])) {
            return false;
        }
        if (empty($_SERVER['HTTP_USER_AGENT'])) {
            return false;
        }
        return true;
    }
    
    /**
     * Attempt: Prevent unallowed Request Methods
     * 
     * @access  private
     * @return  bool
     * @param   string  $rm
     */
    function _RequestMethods($rm)
    {
        if (strlen($rm) < 3) {
            $rm = array('GET', 'POST', 'HEAD');
        } else {
            $rm = array_map('strToUpper', array_map('trim', explode(',', $rm)));
        }
        return in_array(strToUpper($_SERVER['REQUEST_METHOD']), $rm);
    }
    
    /**
     * On Failure
     * 
     * @access  protected
     * @return  bool false
     */
    function onFailure()
    {
        parent::onFailure($this->condition);
        Gadget::logout('/');
        return false;
    }
    
}
?>
