<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Compiler.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/**
 * Requires Gadget_Object
 */
class_exists('Gadget_Object') or require_once 'Gadget/Object.php';

/**
 * Requires Gadget_Template_Action
 */
class_exists('Gadget_Template_Action') or require_once 'Gadget/Template/Action.php';

/**
 * Gadget_Template_Compiler
 *
 * Compiler of Gadget's Template Engine.
 * 
 * @package     Gadget
 * @subpackage  Gadget_Template
 * @since       1.0
 */
class Gadget_Template_Compiler extends Gadget_Object
{
    /**
     * Escape Character
     * 
     * @var string
     */
    var $escapeChar = '\\';
    
    /**
     * Start Character
     * 
     * @var string
     */
    var $startChar  = '{';
    
    /**
     * End Character
     * 
     * @var string
     */
    var $endChar    = '}';
    
    /**
     * Variable Character
     * 
     * @var string
     */
    var $varChar    = '$';
    
    /**
     * Quoted Chars 
     *
     * @var array
     */
    var $quotedChars = array();
    
    /**
     * PCRE Recursion
     * 
     * @var string
     */
    var $pcreRecursion;
    
    /**
     * PCRE Delimiter
     * 
     * @var string
     */
    var $pcreDelimiter = '/';
    
    /**
     * PCRE Patterns
     *
     * @var array
     */
    var $patterns = array();
    
    /**
     * Actions
     * 
     * @var array
     */
    var $actions = array();
    
    /**
     * Parse Xml 
     * 
     * Whether to parse the template as XML.
     * If you won't use the <tag gadget:action="" /> 
     * syntax, disable this for better performance.
     * 
     * @var bool
     */
    var $parseXml = true;
    
    /**
     * Namespace to Parse
     *
     * @var string
     */
    var $parseNs = 'gadget';
    
    /**
     * Whether the XmlParser should strip the comments
     * 
     * @var bool
     */
    var $stripComments = false;
    
    /**
     * XmlParser 
     *
     * @var object Gadget_Template_XmlParser
     */
    var $XmlParser;
    
    /**
     * On After Construct
     * 
     * @access  protected
     * @return  void
     */
    function onAfterConstruct()
    {
        $this->reset();
    }
    
    /**
     * Reset
     * 
     * @access  public
     * @return  void
     */
    function reset()
    {
        $escape = preg_quote($this->escapeChar, $this->pcreDelimiter);
        $start  = preg_quote($this->startChar, $this->pcreDelimiter);
        $end    = preg_quote($this->endChar, $this->pcreDelimiter);
        $var    = $this->varChar == '$' ? "\\\$" : 
            preg_quote($this->varChar, $this->pcreDelimiter);
        
        $this->pcreRecursion = "(((?>[^$start$end]+)|(?R))*)";
        
        $start  = '(?<!'. $escape .')'. $start;
        $end    = '(?<!'. $escape .')'. $end;
        
        $this->quotedChars = compact('escape', 'start', 'end', 'var');
        
        $options = array(
            'pcreDelimiter' => $this->pcreDelimiter,
            'pcreRecursion' => $this->pcreRecursion,
            'quotedChars'   => $this->quotedChars
        );
        
        Gadget_Template_Action::loadActions($actions, $options);
        
        foreach (array_reverse(array_keys($actions)) as $action) {
            $this->actions[$action][0]  = &$actions[$action];
            $this->actions[$action][1]  = $actions[$action]->callback;
            $this->patterns[$action]    = $actions[$action]->composedPattern;
        }
    }
    
    /**
     * Singleton
     * 
     * @access  public
     * @return  object
     */
    function &singleton()
    {
        if (!isset($GLOBALS['_Gadget']['Template']['Compiler']['Instance'])) {
            $GLOBALS['_Gadget']['Template']['Compiler']['Instance'] = 
                &new Gadget_Template_Compiler;
        }
        return $GLOBALS['_Gadget']['Template']['Compiler']['Instance'];
    }
    
    /**
     * Compile
     * 
     * @access  public
     * @return  bool
     * @param   string  $prepared
     * @param   string  $compiled
     */
    function compile($prepared, $compiled)
    {
        $data = file_get_contents($prepared);
        
        if ($this->parseXml) {
            $data = $this->parseXml($data);
        }
        $this->saveTemplate($data, $compiled .'.dbg.php');
        
        $data = $this->parse($data);
        
        return $this->saveTemplate($data, $compiled);
    }
    
    /**
     * Parse
     * 
     * @access  public
     * @return  string
     * @param   string  $data
     */
    function parse($data, $phpize = true)
    {
        foreach ($this->patterns as $a => $pcre) {
            $oldp = $this->actions[$a][0]->phpize;
            $this->actions[$a][0]->phpize = $phpize;
            $data = preg_replace_callback($pcre, $this->actions[$a], $data);
            $this->actions[$a][0]->phpize = $oldp;
        }
        return $data;
    }
    
    /**
     * Parse Xml
     * 
     * @access  protected
     * @return  string
     * @param   string  $data
     */
    function parseXml($data)
    {
        if ($this->XmlParser) {
            $this->XmlParser->reset();
        } else {
            class_exists('Gadget_Template_XmlParser') 
                or require_once 'Gadget/Template/XmlParser.php';
            $this->XmlParser = &new Gadget_Template_XmlParser(
                $this, $this->parseNs);
        }
        
        return $this->XmlParser->parseTemplate($data);
    }
    
    /**
     * Phpize
     * 
     * @internal
     * @access  public
     * @return  string
     * @param   string  $action
     * @param   string  $val
     */
    function phpize($action, $val)
    {
        if (!isset($this->actions[$action])) {
            return null;
        }
        return call_user_func($this->actions[$action], array(null, null, $val));
    }
    
    /**
     * Tplize
     * 
     * @internal
     * @access  public
     * @return  string
     * @param   string  $string, ...
     */
    function tplize($string)
    {
        $strings = func_get_args();
        return 
            $this->startChar . 
            implode($this->endChar . $this->startChar, $strings) . 
            $this->endChar;
    }
    
    /**
     * Varize
     * 
     * @internal
     * @access  public
     * @return  string
     * @param   string  $var
     */
    function varize($var, $phpize = true)
    {
        return $this->parse($var, $phpize);
    }
    
    /**
     * Write Compiled Template
     * 
     * @access  protected
     * @return  bool
     * @param   string  $data
     * @param   string  $file
     */
    function saveTemplate($data, $file)
    {
        if (!is_dir($dir = dirname($file))) {
            class_exists('System') or require_once 'System.php';
            if (!System::mkdir(array('-p', $dir))) {
                return false;
            }
        }
        if (!$fh = @fopen($file, 'w')) {
            return false;
        }
        @flock($fh, LOCK_EX);
        $result = (bool) @fwrite($fh, $data, strlen($data));
        @flock($fh, LOCK_UN);
        @fclose($fh);
        return $result;
    }
}
?>
