<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: FS.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/**
 * Requires Gadget_Tree
 */
class_exists('Gadget_Tree') or require_once 'Gadget/Tree.php';

/**
 * Requires Gadget_Tree_FS_Node
 */
class_exists('Gadget_Tree_FS_Node') or require_once 'Gadget/Tree/FS/Node.php';

/**
 * Gadget_Tree_FS
 * 
 * File system tree
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_FS extends Gadget_Tree 
{
    /**
     * dotFiles
     * 
     * Whether to honor entries like ".", ".." and ".hidden_file".
     * 
     * @access public 
     * @var bool 
     */
    var $dotFiles = true;

    /**
     * onlyFiles
     * 
     * Whether to just include files.
     * 
     * @var bool
     */
    var $onlyFiles = false;
    
    /**
     * onlyDirs
     * 
     * Whether to just include directories.
     * 
     * @var bool
     */
    var $onlyDirs = false;
    
    /**
     * Directory Separator
     * 
     * @var string
     */
    var $dirSep = DIRECTORY_SEPARATOR;
    
    /**
     * Create
     * 
     * See Gadget_Tree::create().
     * 
     * @access  public 
     * @return  object 
     * @param   mixed   $params 
     * @param   string  $class 
     */
    function &create($params, $class = __CLASS__)
    {
        return parent::create($params, $class);
    }

    /**
     * From Default
     * 
     * @access  public 
     * @return  mixed   Returns the root node object on success or false.
     * @param   mixed   $from The source to set/build the tree from.
     */
    function &fromDefault($from)
    {
        return parent::from($from, array('Path' => true));
    }

    /**
     * Build tree from path
     * 
     * Example:
     * <code>
     *     require_once 'Gadget_Tree.php';
     *     $tree = &Gadget_Tree::factory('FS');
     *     $tree->setOptions(array('dotFiles' => false, 'maxDepth' => 3));
     *     $tree->fromPath('.');
     *     $tree->dump();
     * </code>
     * 
     * @access  public 
     * @return  mixed 
     * @param   string $path 
     */
    function fromPath($path)
    {
        if (!is_dir($path = realpath($path))) {
            return Gadget_Tree::raiseError(
                "Cannot create tree from path '$path'.",
                GADGET_TREE_E_INVALID_PATH
            );
        }
        $this->root = &new Gadget_Tree_FS_Node($path);
        $this->root->dirSep = $this->dirSep;
        $this->root->isDir = true;
        $this->node = &$this->root;
        $this->buildTree($path);
        return $this->node = &$this->root;
    }

    /**
     * Builds the directory tree
     * 
     * @access  protected 
     * @return  void 
     * @param   string $path 
     */
    function buildTree($path, $_level = 0)
    {
        if (0 > $this->maxDepth || $this->maxDepth > $_level) {
            $dir = opendir($path);
            while (false !== $entry = readdir($dir)) {
                if ($this->dotFiles || $entry{0} !== '.') {
                    $nPath = $path . $this->dirSep . $entry;
                    $isRef = ($entry === '.' || $entry === '..');
                    $isDir = $isRef || is_dir($nPath);
                    if ($isDir) {
                        if (!$this->onlyFiles) {
                            if ($isRef) {
                                $this->node->addNewChild($entry, true);
                            } else {
                                $this->node = &$this->node->addNewChild($entry, true);
                                $this->buildTree($nPath, $_level + 1);
                            }
                        }
                    } elseif (!$this->onlyDirs) {
                        $this->node->addNewChild($entry, false);
                    }
                }
            }
            closedir($dir);
        }
        if (isset($this->node->parent)) {
            $this->node = &$this->node->parent;
        }
    }
}

?>