<?php

/**
 * Gadget Web Development Kit
 * 
 * @GADGET_HEADER@
 * 
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @author      Michael Wallner <mike@iworks.at>
 * @version     $Revision: 1.1 $
 * 
 * @copyright   2004 Michael Wallner
 * @license     http://www.gnu.org/copyleft/gpl.txt GNU Public License
 * 
 * @cvs         $Id: Search.php,v 1.1 2005/04/04 11:50:58 mike Exp $
 */

/** 
 * Gadget_Tree_Search
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search
{
    /**
     * Path
     * 
     * @var string
     */
    var $path = '';
    
    /**
     * Separator
     * 
     * @var string
     */
    var $sep  = '';
    
    /**
     * Conditions
     * 
     * @var array
     */
    var $conditions = array();
    
    /**
     * Constructor
     * 
     * @param   string  $path
     * @param   string  $sep
     */
    function Gadget_Tree_Search($path, $sep = '/')
    {
        $this->sep = $sep;
        $this->path = $path;
    }
    
    /**
     * Add Condition
     * 
     * @access  public
     * @return  bool
     * @param   object  Gadget_Tree_Search_Condition $condition
     */
    function addCondition(&$condition)
    {
        if ($isCond = is_a($condition, 'Gadget_Tree_Search_Condition')) {
            $this->conditions[] = &$condition;
        }
        return $isCond;
    }
    
    /**
     * Parse
     * 
     * @access  public
     * @return  void
     */
    function parse()
    {
        $parts = explode($this->sep, $this->path);
        for ($i = 1, $p = count($parts); $i < $p; $i++) {
            $c = &new Gadget_Tree_Search_Condition;
            if (preg_match_all('/\[([^\[\]]+)\]/', $parts[$i], $matches)) {
                $c->parseNodeConditions($matches[1]);
            }
            $c->parseNodeAxis(substr($parts[$i], 0, strcspn($parts[$i], '[(')));
            $this->addCondition($c);
        }
    }
    
    /**
     * Search
     * 
     * @access  public
     * @return  mixed
     * @param   object  Gadget_Tree_Node $node
     */
    function &search(&$node)
    {
        $test = &$node;
        for ($i = 0, $c = count($this->conditions); $i < $c; $i++) {
            if (!$test = &$this->conditions[$i]->findNode($test)) {
                return null;
            }
        }
        return $test;
    }
    
    /**
     * Compare
     * 
     * @static
     * @access  public
     * @return  bool
     * @param   string  $op
     * @param   mixed   $left
     * @param   mixed   $right
     */
    function compare($op, $left, $right)
    {
        #echo "[$left] [$op] [$right]\n";
        switch ($op)
        {
            case '=':   return $left == $right;
            case '>':   return $left > $right;
            case '<':   return $left < $right;
            case '>=':  return $left >= $right;
            case '<=':  return $left <= $right;
            case '!=':  return $left != $right;
            default:
                Gadget_Tree::raiseError("Invalid comparison operator $op");
                return false;
        }
    }
    
}

/** 
 * Gadget_Tree_Search_Condition
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search_Condition
{
    /**
     * Node Tests
     * 
     * @var array
     */
    var $nodeTests = array();
    
    /**
     * Axis Tests
     * 
     * @var array
     */
    var $axisTests = array();
    
    /**
     * Parse Node Conditions
     * 
     * @access  public
     * @return  void
     * @param   array   $conditions
     */
    function parseNodeConditions($conditions)
    {
        foreach ((array) $conditions as $c) {
            $attribute = $c{0} == '@';
            $this->nodeTests[] = &new Gadget_Tree_Search_NodeTest(
                $attribute ? 'attribute' : 'function', array_map('trim', 
                preg_split('/([!=><]+)/', $c, 2, PREG_SPLIT_DELIM_CAPTURE))
            );
        }
    }
    
    /**
     * Parse Node Axis
     * 
     * @access  public
     * @return  void
     * @param   string  $axis
     */
    function parseNodeAxis($axis)
    {
        $this->axisTests[] = &new Gadget_Tree_Search_AxisTest($axis);
    }
    
    /**
     * Test Node
     * 
     * @access  public
     * @return  mixed
     * @param   object  Gadget_Tree_Node $node
     */
    function testNode(&$node)
    {
        for ($i = 0, $c = count($this->axisTests); $i < $c; $i++) {
            if (!$node = &$this->axisTests[$i]->testNode($node)) {
                return null;
            }
        }
        for ($i = 0, $c = count($this->nodeTests); $i < $c; $i++) {
            if (!$node = &$this->nodeTests[$i]->testNode($node)) {
                return null;
            }
        }
        return $node;
    }
    
    /**
     * Find Node
     * 
     * @access  public
     * @return  mixed
     * @param   object Gadget_Tree_Node $node
     */
    function findNode(&$node)
    {
        static $rootTested = false;
        
        if (!$rootTested) {
            $rootTested = true;
            if ($root = &$this->testNode($node)) {
                return $root;
            }
        }
        
        for ($i = 0, $c = $node->count(); $i < $c; $i++) {
            if ($child = &$this->testNode($node->childAt($i))) {
                return $child;
            }
        }
        
        $s = &$node->followingSibling();
        for ($i = 0, $c = count($s); $i < $c; $i++) {
            if (    $child = &$this->testNode($s[$i]) && 
                    $child = &$this->findNode($child)) {
                return $child;
            }
        }
        return null;
    }
}

/** 
 * Gadget_Tree_Search_NodeTest
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search_NodeTest
{
    /**
     * Type
     * 
     * @var string
     */
    var $type;
    
    /**
     * Operator
     * 
     * @var string
     */
    var $operator;
    
    /**
     * Left Operand
     * 
     * @var string
     */
    var $L;
    
    /**
     * Right Operand
     * 
     * @var string
     */
    var $R;
    
    /**
     * Constructor
     * 
     * @param   string  $type
     * @param   array   $test
     */
    function Gadget_Tree_Search_NodeTest($type, $test)
    {
        $this->type = strToLower($type);
        $this->operator = $test[1];
        
        #print_r($test);
        
        switch ($this->type)
        {
            case 'attribute':
                $this->L = substr($test[0], 1);
                $this->R = $test[2];
            break;
            
            case 'function':
                $this->L = $test[0];
                $this->R = $test[2];
            break;
            
            default:
                Gadget_Tree::raiseError("Not a valid NodeTest type: $type");
            break;
        }
    }
    
    /**
     * Test Node
     * 
     * @access  public
     * @return  mixed
     * @param   object  Gadget_Tree_Node $node
     */
    function &testNode(&$node)
    {
        #echo "Testing {$this->type} {$this->L} on/of {$node->name} to be {$this->R} ";
        $test = false;
        switch ($this->type)
        {
            case 'attribute':
                #echo $node->getMeta($this->L), "\n";
                $test = Gadget_Tree_Search::compare($this->operator,
                    $node->getMeta($this->L), trim($this->R, '"'));
            break;
            
            case 'function':
                switch ($this->L)
                {
                    case 'position()':
                        #echo $node->position(), "\n";
                        $test = Gadget_Tree_Search::compare($this->operator,
                            $node->position(), $this->R);
                    break;
                }
            break;
        }
        return $test ? $node : null;
    }
}

/** 
 * Gadget_Tree_Search_AxisTest
 *
 * @package     Gadget
 * @subpackage  Gadget_Tree
 * @since       1.0
 */
class Gadget_Tree_Search_AxisTest
{
    /**
     * Axis
     * 
     * @var string
     */
    var $axis;
    
    /**
     * Constructor
     * 
     * @param   string  $axis
     */
    function Gadget_Tree_Search_AxisTest($axis)
    {
        $this->axis = $axis;
    }
    
    /**
     * Test Node
     * 
     * @access  public
     * @return  mixed
     * @param   object  Gadget_Tree_Node $node
     */
    function &testNode(&$node)
    {
        #echo "Testing {$node->name} to be {$this->axis}\n";
        if ($this->axis == '*') {
            return $node;
        }
        return $node->name == $this->axis ? $node : null;
    }
}

?>
